@@ -12,181 +12,18 @@ except ImportError:
     from urllib import urlencode
 
 
-class Client(object):
-    """Quickly and easily access any REST or REST-like API."""
-    def __init__(self,
-                 host,
-                 request_headers=None,
-                 version=None):
+class Response(object):
+    """Holds the response from an API call."""
+    def __init__(self, response):
         """
-        :param host: Base URL for the api. (e.g. https://api.sendgrid.com)
-        :type host:  string
-        :param request_headers: A dictionary of the headers you want
-                                applied on all calls
-        :type request_headers: dictionary
-        :param version: The version number of the API.
-                        Subclass _build_versioned_url for custom behavior.
-                        Or just pass the version as part of the URL
-                        (e.g. client._("/v3"))
-        :type integer:
-        """
-        self.host = host
-        self.request_headers = request_headers
-        # These are the supported HTTP verbs
-        self.methods = ['delete', 'get', 'patch', 'post', 'put']
-        self._version = version
-        # _count and _url_path keep track of the dynamically built url
-        self._count = 0
-        self._url_path = {}
-        self._status_code = None
-        self._response_body = None
-        self._response_headers = None
-        self._response = None
-
-    def _reset(self):
-        """Resets the URL builder, so you can make a fresh new dynamic call."""
-        self._count = 0
-        self._url_path = {}
-        self._response = None
-
-    def _add_to_url_path(self, name):
-        """Takes the method chained call and adds to the url path.
-
-           :param name: The name of the method call
-           :type name: string
-        """
-        self._url_path[self._count] = name
-        self._count += 1
-
-    def _build_versioned_url(self, url):
-        """Subclass this function for your own needs.
-           Or just pass the version as part of the URL
-           (e.g. client._('/v3'))
-        :param url: URI portion of the full URL being requested
-        :type url: string
-        :return:
-        """
-        return '{0}/v{1}{2}'.format(self.host, str(self._version), url)
-
-    def _build_url(self, query_params):
-        """Build the final URL to be passed to urllib
-
-        :param query_params: A dictionary of all the query parameters
-        :type query_params: dictionary
-        :return:
-        """
-        url = ''
-        count = 0
-        while count < len(self._url_path):
-            url += '/{0}'.format(self._url_path[count])
-            count += 1
-        if query_params:
-            url_values = urlencode(sorted(query_params.items()))
-            url = '{0}?{1}'.format(url, url_values)
-        if self._version:
-            url = self._build_versioned_url(url)
-        else:
-            url = self.host + url
-        return url
-
-    def _set_response(self, response):
-        """Build the API call's response
-
-        :param response: The response object from the API call from urllib
-        :type response: urllib.Request object
+        :param response: The return value from a open call
+                         on a urllib.build_opener()
+        :type response:  urllib response object
         """
         self._status_code = response.getcode()
         self._response_body = response.read()
         self._response_headers = response.info()
 
-    def _set_headers(self, request_headers):
-        """Build the headers for the request
-
-        :param request_headers: headers to set for the API call
-        :type response: dict
-        :return:
-        """
-        if self.request_headers:
-            self.request_headers.update(request_headers)
-        else:
-            self.request_headers = request_headers
-
-    def _make_request(self, opener, request):
-        """Make the API call and return the response. This is separated into
-           it's own function, so we can mock it easily for testing.
-
-        :param opener:
-        :type opener:
-        :param request: url payload to request
-        :type request: urllib.Request object
-        :return:
-        """
-        return opener.open(request)
-
-    def _(self, name):
-        """Add variable values to the url.
-           (e.g. /your/api/{variable_value}/call)
-           Another example: if you have a Python reserved word, such as global,
-           in your url, you must use this method.
-
-        :param name: Name of the url segment
-        :type name: string
-        :return:
-        """
-        self._add_to_url_path(name)
-        return self
-
-    def __getattr__(self, name):
-        """Dynamically add method calls to the url, then call a method.
-           (e.g. client.name.name.method())
-           You can also add a version number by using .version(<int>)
-
-        :param name: Name of the url segment or method call
-        :type name: string or integer if name == version
-        :return: mixed
-        """
-        if name == 'version':
-            def get_version(*args, **kwargs):
-                """
-                :param args: dict of settings
-                :param kwargs: unused
-                :return: string, version
-                """
-                self._version = args[0]
-                return self
-            return get_version
-
-        # We have reached the end of the method chain, make the API call
-        if name in self.methods:
-            method = name.upper()
-
-            def http_request(*args, **kwargs):
-                """Make the API call
-                :param args: unused
-                :param kwargs:
-                :return: Client object
-                """
-                if 'request_headers' in kwargs:
-                    self._set_headers(kwargs['request_headers'])
-                data = json.dumps(kwargs['request_body']).encode('utf-8')\
-                    if 'request_body' in kwargs else None
-                params = kwargs['query_params']\
-                    if 'query_params' in kwargs else None
-                opener = urllib.build_opener()
-                request = urllib.Request(self._build_url(params), data=data)
-                if self.request_headers:
-                    for key, value in self.request_headers.items():
-                        request.add_header(key, value)
-                request.get_method = lambda: method
-                self._response = self._make_request(opener, request)
-                self._set_response(self._response)
-                self._reset()
-                return self
-            return http_request
-        else:
-            self._add_to_url_path(name)
-        return self
-
     @property
     def status_code(self):
         """
@@ -207,3 +44,155 @@ class Client(object):
         :return: dict of response headers
         """
         return self._response_headers
+
+
+class Client(object):
+    """Quickly and easily access any REST or REST-like API."""
+    def __init__(self,
+                 host,
+                 request_headers=None,
+                 version=None,
+                 url_path=None):
+        """
+        :param host: Base URL for the api. (e.g. https://api.sendgrid.com)
+        :type host:  string
+        :param request_headers: A dictionary of the headers you want
+                                applied on all calls
+        :type request_headers: dictionary
+        :param version: The version number of the API.
+                        Subclass _build_versioned_url for custom behavior.
+                        Or just pass the version as part of the URL
+                        (e.g. client._("/v3"))
+        :type version: integer
+        :param url_path: A list of the url path segments
+        :type url_path: list of strings
+        """
+        self.host = host
+        self.request_headers = request_headers or {}
+        self._version = version
+        # _url_path keeps track of the dynamically built url
+        self._url_path = url_path or []
+        # These are the supported HTTP verbs
+        self.methods = ['delete', 'get', 'patch', 'post', 'put']
+
+    def _build_versioned_url(self, url):
+        """Subclass this function for your own needs.
+           Or just pass the version as part of the URL
+           (e.g. client._('/v3'))
+        :param url: URI portion of the full URL being requested
+        :type url: string
+        :return: string
+        """
+        return '{0}/v{1}{2}'.format(self.host, str(self._version), url)
+
+    def _build_url(self, query_params):
+        """Build the final URL to be passed to urllib
+
+        :param query_params: A dictionary of all the query parameters
+        :type query_params: dictionary
+        :return: string
+        """
+        url = ''
+        count = 0
+        while count < len(self._url_path):
+            url += '/{0}'.format(self._url_path[count])
+            count += 1
+        if query_params:
+            url_values = urlencode(sorted(query_params.items()))
+            url = '{0}?{1}'.format(url, url_values)
+        url = self._build_versioned_url(url) if self._version else self.host + url
+        return url
+
+    def _update_headers(self, request_headers):
+        """Update the headers for the request
+
+        :param request_headers: headers to set for the API call
+        :type response: dictionary
+        :return: dictionary
+        """
+        self.request_headers.update(request_headers)
+
+    def _build_client(self, name=None):
+        """Make a new Client object
+
+        :param name: Name of the url segment
+        :type name: string
+        :return: A Client object
+        """
+        url_path = self._url_path+[name] if name else self._url_path
+        return Client(host=self.host,
+                      version=self._version,
+                      request_headers=self.request_headers,
+                      url_path=url_path)
+
+    def _make_request(self, opener, request):
+        """Make the API call and return the response. This is separated into
+           it's own function, so we can mock it easily for testing.
+
+        :param opener:
+        :type opener:
+        :param request: url payload to request
+        :type request: urllib.Request object
+        :return: urllib response
+        """
+        return opener.open(request)
+
+    def _(self, name):
+        """Add variable values to the url.
+           (e.g. /your/api/{variable_value}/call)
+           Another example: if you have a Python reserved word, such as global,
+           in your url, you must use this method.
+
+        :param name: Name of the url segment
+        :type name: string
+        :return: Client object
+        """
+        return self._build_client(name)
+
+    def __getattr__(self, name):
+        """Dynamically add method calls to the url, then call a method.
+           (e.g. client.name.name.method())
+           You can also add a version number by using .version(<int>)
+
+        :param name: Name of the url segment or method call
+        :type name: string or integer if name == version
+        :return: mixed
+        """
+        if name == 'version':
+            def get_version(*args, **kwargs):
+                """
+                :param args: dict of settings
+                :param kwargs: unused
+                :return: string, version
+                """
+                self._version = args[0]
+                return self._build_client()
+            return get_version
+
+        # We have reached the end of the method chain, make the API call
+        if name in self.methods:
+            method = name.upper()
+
+            def http_request(*args, **kwargs):
+                """Make the API call
+                :param args: unused
+                :param kwargs:
+                :return: Client object
+                """
+                if 'request_headers' in kwargs:
+                    self._update_headers(kwargs['request_headers'])
+                data = json.dumps(kwargs['request_body']).encode('utf-8')\
+                    if 'request_body' in kwargs else None
+                params = kwargs['query_params']\
+                    if 'query_params' in kwargs else None
+                opener = urllib.build_opener()
+                request = urllib.Request(self._build_url(params), data=data)
+                if self.request_headers:
+                    for key, value in self.request_headers.items():
+                        request.add_header(key, value)
+                request.get_method = lambda: method
+                return Response(self._make_request(opener, request))
+            return http_request
+        else:
+            # Add a segment to the URL
+            return self._(name)
